@@ -1,6 +1,7 @@
 /*
  * SPDX-FileCopyrightText: 2024 Bohdan Onofriichuk <bogdan.onofriuchuk@gmail.com>
  * SPDX-FileCopyrightText: 2024 Natalie Clarius <natalie.clarius@kde.org>
+ * SPDX-FileCopyrightText: 2025 Jakob Petsovits <jpetso@petsovits.com>
 
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
@@ -15,6 +16,7 @@
 #include <QDBusMetaType>
 #include <QDBusReply>
 #include <QDBusServiceWatcher>
+#include <QDBusVariant>
 #include <QString>
 #include <QVariant>
 
@@ -22,16 +24,40 @@
 
 #include "inhibitmonitor_p.h"
 
+using namespace Qt::Literals::StringLiterals;
+
 static constexpr QLatin1StringView SOLID_POWERMANAGEMENT_SERVICE("org.kde.Solid.PowerManagement");
 static constexpr QLatin1StringView FDO_POWERMANAGEMENT_SERVICE("org.freedesktop.PowerManagement");
 
+static constexpr QLatin1StringView POLICY_AGENT_PATH("/org/kde/Solid/PowerManagement/PolicyAgent");
+static constexpr QLatin1StringView POLICY_AGENT_IFACE("org.kde.Solid.PowerManagement.PolicyAgent");
+
+Q_DECLARE_METATYPE(PolicyAgentInhibition)
+Q_DECLARE_METATYPE(QList<PolicyAgentInhibition>)
+
+QDBusArgument &operator<<(QDBusArgument &argument, const PolicyAgentInhibition &inhibition)
+{
+    argument.beginStructure();
+    argument << inhibition.what << inhibition.who << inhibition.why << inhibition.mode << inhibition.flags;
+    argument.endStructure();
+    return argument;
+}
+
+const QDBusArgument &operator>>(const QDBusArgument &argument, PolicyAgentInhibition &inhibition)
+{
+    argument.beginStructure();
+    argument >> inhibition.what >> inhibition.who >> inhibition.why >> inhibition.mode >> inhibition.flags;
+    argument.endStructure();
+    return argument;
+}
+
 InhibitionControl::InhibitionControl(QObject *parent)
     : QObject(parent)
     , m_solidWatcher(new QDBusServiceWatcher)
     , m_fdoWatcher(new QDBusServiceWatcher)
 {
-    qDBusRegisterMetaType<QList<InhibitionInfo>>();
-    qDBusRegisterMetaType<InhibitionInfo>();
+    qDBusRegisterMetaType<PolicyAgentInhibition>();
+    qDBusRegisterMetaType<QList<PolicyAgentInhibition>>();
 
     // Watch for PowerDevil's power management service
     m_solidWatcher->setConnection(QDBusConnection::sessionBus());
@@ -117,9 +143,7 @@ void InhibitionControl::onServiceRegistered(const QString &serviceName)
             watcher->deleteLater();
         });
 
-        onInhibitionsChanged({}, {});
-        onPermanentlyBlockedInhibitionsChanged({}, {});
-        onTemporarilyBlockedInhibitionsChanged({}, {});
+        checkInhibitions();
 
         QDBusMessage hasInhibitMessage = QDBusMessage::createMethodCall(SOLID_POWERMANAGEMENT_SERVICE,
                                                                         QStringLiteral("/org/freedesktop/PowerManagement"),
@@ -138,28 +162,15 @@ void InhibitionControl::onServiceRegistered(const QString &serviceName)
         });
 
         if (!QDBusConnection::sessionBus().connect(SOLID_POWERMANAGEMENT_SERVICE,
-                                                   QStringLiteral("/org/kde/Solid/PowerManagement/PolicyAgent"),
-                                                   QStringLiteral("org.kde.Solid.PowerManagement.PolicyAgent"),
-                                                   QStringLiteral("InhibitionsChanged"),
+                                                   POLICY_AGENT_PATH,
+                                                   u"org.freedesktop.DBus.Properties"_s,
+                                                   u"PropertiesChanged"_s,
                                                    this,
-                                                   SLOT(onInhibitionsChanged(QList<InhibitionInfo>, QStringList)))) {
-            qCDebug(APPLETS::BATTERYMONITOR) << "Error connecting to inhibition changes via dbus";
-        }
-        if (!QDBusConnection::sessionBus().connect(SOLID_POWERMANAGEMENT_SERVICE,
-                                                   QStringLiteral("/org/kde/Solid/PowerManagement/PolicyAgent"),
-                                                   QStringLiteral("org.kde.Solid.PowerManagement.PolicyAgent"),
-                                                   QStringLiteral("PermanentlyBlockedInhibitionsChanged"),
-                                                   this,
-                                                   SLOT(onPermanentlyBlockedInhibitionsChanged(QList<InhibitionInfo>, QList<InhibitionInfo>)))) {
-            qCDebug(APPLETS::BATTERYMONITOR) << "Error connecting to permanently blocked inhibition changes via dbus";
-        }
-        if (!QDBusConnection::sessionBus().connect(SOLID_POWERMANAGEMENT_SERVICE,
-                                                   QStringLiteral("/org/kde/Solid/PowerManagement/PolicyAgent"),
-                                                   QStringLiteral("org.kde.Solid.PowerManagement.PolicyAgent"),
-                                                   QStringLiteral("TemporarilyBlockedInhibitionsChanged"),
-                                                   this,
-                                                   SLOT(onTemporarilyBlockedInhibitionsChanged(QList<InhibitionInfo>, QList<InhibitionInfo>)))) {
-            qCDebug(APPLETS::BATTERYMONITOR) << "Error connecting to temporarily blocked inhibition changes via dbus";
+                                                   // clang-format off
+                                                   SLOT(onPolicyAgentPropertiesChanged(QString,QVariantMap,QStringList))
+                                                   // clang-format on
+                                                   )) {
+            qCDebug(APPLETS::BATTERYMONITOR) << "Error connecting to PolicyAgent inhibition changes via dbus";
         }
     }
 }
@@ -184,26 +195,13 @@ void InhibitionControl::onServiceUnregistered(const QString &serviceName)
                                                  this,
                                                  SLOT(triggersLidActionChanged(bool)));
         QDBusConnection::sessionBus().disconnect(SOLID_POWERMANAGEMENT_SERVICE,
-                                                 QStringLiteral("/org/kde/Solid/PowerManagement/PolicyAgent"),
-                                                 QStringLiteral("org.kde.Solid.PowerManagement.PolicyAgent"),
-                                                 QStringLiteral("InhibitionsChanged"),
+                                                 POLICY_AGENT_PATH,
+                                                 u"org.freedesktop.DBus.Properties"_s,
+                                                 u"PropertiesChanged"_s,
                                                  this,
-                                                 SLOT(onInhibitionsChanged(QList<InhibitionInfo>, QStringList)));
-        QDBusConnection::sessionBus().disconnect(SOLID_POWERMANAGEMENT_SERVICE,
-                                                 QStringLiteral("/org/kde/Solid/PowerManagement/PolicyAgent"),
-                                                 QStringLiteral("org.kde.Solid.PowerManagement.PolicyAgent"),
-                                                 QStringLiteral("PermanentlyBlockedInhibitionsChanged"),
-                                                 this,
-                                                 SLOT(onPermanentlyBlockedInhibitionsChanged(QList<InhibitionInfo>, QList<InhibitionInfo>)));
-        QDBusConnection::sessionBus().disconnect(SOLID_POWERMANAGEMENT_SERVICE,
-                                                 QStringLiteral("/org/kde/Solid/PowerManagement/PolicyAgent"),
-                                                 QStringLiteral("org.kde.Solid.PowerManagement.PolicyAgent"),
-                                                 QStringLiteral("TemporarilyBlockedInhibitionsChanged"),
-                                                 this,
-                                                 SLOT(onTemporarilyBlockedInhibitionsChanged(QList<InhibitionInfo>, QList<InhibitionInfo>)));
+                                                 SLOT(onPolicyAgentPropertiesChanged(QString, QVariantMap, QStringList)));
 
-        m_inhibitions = QList<QVariantMap>();
-        m_blockedInhibitions = QList<QVariantMap>();
+        m_requestedInhibitions = QList<RequestedInhibition>{};
         m_hasInhibition = false;
         m_isManuallyInhibited = false;
         m_isManuallyInhibitedError = false;
@@ -222,25 +220,15 @@ void InhibitionControl::uninhibit()
     InhibitMonitor::self().uninhibit(m_isSilent);
 }
 
-void InhibitionControl::blockInhibition(const QString &appName, const QString &reason, bool permanently)
+void InhibitionControl::setInhibitionAllowed(const QString &appName, const QString &reason, bool allowed)
 {
-    qDebug() << "Blocking inhibition for" << appName << "with reason" << reason << (permanently ? "permanently" : "temporarily");
-    QDBusMessage msg = QDBusMessage::createMethodCall(SOLID_POWERMANAGEMENT_SERVICE,
-                                                      QStringLiteral("/org/kde/Solid/PowerManagement/PolicyAgent"),
-                                                      QStringLiteral("org.kde.Solid.PowerManagement.PolicyAgent"),
-                                                      QStringLiteral("BlockInhibition"));
-    msg << appName << reason << permanently;
-    QDBusPendingCall call = QDBusConnection::sessionBus().asyncCall(msg);
-}
-
-void InhibitionControl::unblockInhibition(const QString &appName, const QString &reason, bool permanently)
-{
-    qDebug() << "Unblocking inhibition for" << appName << "with reason" << reason << (permanently ? "permanently" : "temporarily");
-    QDBusMessage msg = QDBusMessage::createMethodCall(SOLID_POWERMANAGEMENT_SERVICE,
-                                                      QStringLiteral("/org/kde/Solid/PowerManagement/PolicyAgent"),
-                                                      QStringLiteral("org.kde.Solid.PowerManagement.PolicyAgent"),
-                                                      QStringLiteral("UnblockInhibition"));
-    msg << appName << reason << permanently;
+    if (allowed) {
+        qCDebug(APPLETS::BATTERYMONITOR) << "Allowing inhibition for" << appName << "with reason" << reason;
+    } else {
+        qCDebug(APPLETS::BATTERYMONITOR) << "Suppressing inhibition for" << appName << "with reason" << reason;
+    }
+    QDBusMessage msg = QDBusMessage::createMethodCall(SOLID_POWERMANAGEMENT_SERVICE, POLICY_AGENT_PATH, POLICY_AGENT_IFACE, u"SetInhibitionAllowed"_s);
+    msg << appName << reason << allowed;
     QDBusPendingCall call = QDBusConnection::sessionBus().asyncCall(msg);
 }
 
@@ -254,14 +242,9 @@ void InhibitionControl::setIsSilent(bool status)
     m_isSilent = status;
 }
 
-QBindable<QList<QVariantMap>> InhibitionControl::bindableInhibitions()
+QBindable<QList<RequestedInhibition>> InhibitionControl::bindableRequestedInhibitions()
 {
-    return &m_inhibitions;
-}
-
-QBindable<QList<QVariantMap>> InhibitionControl::bindableBlockedInhibitions()
-{
-    return &m_blockedInhibitions;
+    return &m_requestedInhibitions;
 }
 
 QBindable<bool> InhibitionControl::bindableHasInhibition()
@@ -289,38 +272,6 @@ QBindable<bool> InhibitionControl::bindableIsManuallyInhibitedError()
     return &m_isManuallyInhibitedError;
 }
 
-void InhibitionControl::onInhibitionsChanged(const QList<InhibitionInfo> &added, const QStringList &removed)
-{
-    Q_UNUSED(added);
-    Q_UNUSED(removed);
-
-    QDBusMessage inhibitionsMessage = QDBusMessage::createMethodCall(SOLID_POWERMANAGEMENT_SERVICE,
-                                                                     QStringLiteral("/org/kde/Solid/PowerManagement/PolicyAgent"),
-                                                                     QStringLiteral("org.kde.Solid.PowerManagement.PolicyAgent"),
-                                                                     QStringLiteral("ListInhibitions"));
-    QDBusPendingCall inhibitionsCall = QDBusConnection::sessionBus().asyncCall(inhibitionsMessage);
-    auto inhibitionsWatcher = new QDBusPendingCallWatcher(inhibitionsCall, this);
-    connect(inhibitionsWatcher, &QDBusPendingCallWatcher::finished, this, [this](QDBusPendingCallWatcher *watcher) {
-        QDBusReply<QList<InhibitionInfo>> reply = *watcher;
-        if (reply.isValid()) {
-            updateInhibitions(reply.value());
-        } else {
-            qCDebug(APPLETS::BATTERYMONITOR) << "Failed to retrive inhibitions";
-        }
-        watcher->deleteLater();
-    });
-}
-
-void InhibitionControl::onPermanentlyBlockedInhibitionsChanged(const QList<InhibitionInfo> &added, const QList<InhibitionInfo> &removed)
-{
-    updateBlockedInhibitions(added, removed, {}, {});
-}
-
-void InhibitionControl::onTemporarilyBlockedInhibitionsChanged(const QList<InhibitionInfo> &added, const QList<InhibitionInfo> &removed)
-{
-    updateBlockedInhibitions({}, {}, added, removed);
-}
-
 void InhibitionControl::onHasInhibitionChanged(bool status)
 {
     m_hasInhibition = status;
@@ -336,66 +287,70 @@ void InhibitionControl::onisManuallyInhibitedErrorChanged(bool status)
     m_isManuallyInhibitedError = status;
 }
 
-void InhibitionControl::updateInhibitions(const QList<InhibitionInfo> &inhibitions)
+void InhibitionControl::onPolicyAgentPropertiesChanged(const QString &ifaceName, const QVariantMap &changedProps, const QStringList &invalidatedProps)
 {
-    QList<QVariantMap> out;
+    const QString inhibitions = u"RequestedInhibitions"_s;
 
-    for (auto it = inhibitions.constBegin(); it != inhibitions.constEnd(); ++it) {
-        const QString &name = (*it).first;
-        if (name == QStringLiteral("plasmashell") || name == QStringLiteral("org.kde.plasmashell")) {
-            continue;
+    if (ifaceName == POLICY_AGENT_IFACE) {
+        if (changedProps.contains(inhibitions) || invalidatedProps.contains(inhibitions)) {
+            checkInhibitions();
         }
-        QString prettyName;
-        QString icon;
-        const QString &reason = (*it).second;
-
-        m_data.populateApplicationData(name, &prettyName, &icon);
-
-        out.append(QVariantMap{{QStringLiteral("Name"), name},
-                               {QStringLiteral("PrettyName"), prettyName},
-                               {QStringLiteral("Icon"), icon},
-                               {QStringLiteral("Reason"), reason}});
     }
-
-    m_inhibitions = out;
 }
 
-void InhibitionControl::updateBlockedInhibitions(const QList<InhibitionInfo> &permanentlyBlockedAdded,
-                                                 const QList<InhibitionInfo> &permanentlyBlockedRemoved,
-                                                 const QList<InhibitionInfo> &temporarilyBlockedAdded,
-                                                 const QList<InhibitionInfo> &temporarilyBlockedRemoved)
+void InhibitionControl::checkInhibitions()
 {
-    auto blockedInhibition = [this](const InhibitionInfo &item, const bool permanently) {
-        const QString &name = item.first;
+    QDBusMessage msg = QDBusMessage::createMethodCall(SOLID_POWERMANAGEMENT_SERVICE, POLICY_AGENT_PATH, u"org.freedesktop.DBus.Properties"_s, u"Get"_s);
+    msg << POLICY_AGENT_IFACE << u"RequestedInhibitions"_s;
+    QDBusPendingCall call = QDBusConnection::sessionBus().asyncCall(msg);
+
+    auto watcher = new QDBusPendingCallWatcher(call, this);
+    connect(watcher, &QDBusPendingCallWatcher::finished, this, [this](QDBusPendingCallWatcher *watcher) {
+        QDBusReply<QDBusVariant> reply = *watcher;
+        if (reply.isValid()) {
+            // "Get" returns a D-Bus VARIANT of an array of structures that contain PolicyAgentInhibition data.
+            // But the variant doesn't know that this is a QList<PolicyAgentInhibition>, so instead
+            // get the untyped value of the VARIANT and use our custom D-Bus operator>>() to convert it.
+            m_requestedInhibitions = updatedInhibitions(qdbus_cast<QList<PolicyAgentInhibition>>(reply.value().variant().value<QDBusArgument>()));
+        } else {
+            qCWarning(APPLETS::BATTERYMONITOR) << "Failed to retrieve inhibitions:" << reply.error().message();
+        }
+        watcher->deleteLater();
+    });
+}
+
+QList<RequestedInhibition> InhibitionControl::updatedInhibitions(const QList<PolicyAgentInhibition> &inhibitions)
+{
+    QList<RequestedInhibition> result;
+
+    for (const PolicyAgentInhibition &inhibition : inhibitions) {
+        if (inhibition.who == "plasmashell"_L1 || inhibition.who == "org.kde.plasmashell"_L1) {
+            continue;
+        }
+        if (inhibition.mode != "block"_L1) {
+            continue;
+        }
+        const QStringList whats = inhibition.what.split(":"_L1);
+        if (!whats.contains(u"sleep"_s) && !whats.contains(u"idle"_s)) {
+            continue; // maybe we'll support other inhibition types at a later point
+        }
+
         QString prettyName;
         QString icon;
-        const QString &reason = item.second;
+        m_data.populateApplicationData(inhibition.who, &prettyName, &icon);
 
-        m_data.populateApplicationData(name, &prettyName, &icon);
-
-        return QVariantMap{{QStringLiteral("Name"), name},
-                           {QStringLiteral("PrettyName"), prettyName},
-                           {QStringLiteral("Icon"), icon},
-                           {QStringLiteral("Reason"), reason},
-                           {QStringLiteral("Permanently"), permanently}};
-    };
-
-    auto out = InhibitionControl::bindableBlockedInhibitions().value();
-
-    for (auto it = permanentlyBlockedAdded.constBegin(); it != permanentlyBlockedAdded.constEnd(); ++it) {
-        out.append(blockedInhibition(*it, true));
-    }
-    for (auto it = permanentlyBlockedRemoved.constBegin(); it != permanentlyBlockedRemoved.constEnd(); ++it) {
-        out.removeOne(blockedInhibition(*it, true));
-    }
-    for (auto it = temporarilyBlockedAdded.constBegin(); it != temporarilyBlockedAdded.constEnd(); ++it) {
-        out.append(blockedInhibition(*it, false));
-    }
-    for (auto it = temporarilyBlockedRemoved.constBegin(); it != temporarilyBlockedRemoved.constEnd(); ++it) {
-        out.removeOne(blockedInhibition(*it, false));
+        result.append(RequestedInhibition{
+            .appName = inhibition.who,
+            .prettyName = prettyName,
+            .icon = icon,
+            .reason = inhibition.why,
+            .what = whats,
+            .active = static_cast<bool>(inhibition.flags & PolicyAgentInhibition::Active),
+            .allowed = static_cast<bool>(inhibition.flags & PolicyAgentInhibition::Allowed),
+        });
     }
 
-    m_blockedInhibitions = out;
+    return result;
 }
 
 #include "moc_inhibitioncontrol.cpp"
