@@ -23,7 +23,7 @@
 #include <sys/socket.h>
 
 static constexpr QLatin1StringView SOLID_POWERMANAGEMENT_SERVICE("org.kde.Solid.PowerManagement");
-static constexpr QLatin1StringView UPOWER_POWERPROFILE_SERVICE("net.hadess.PowerProfiles");
+static constexpr QLatin1StringView UPOWER_POWERPROFILE_SERVICE("org.freedesktop.UPower.PowerProfiles");
 
 PowerProfilesControl::PowerProfilesControl(QObject *parent)
     : QObject(parent)
@@ -423,3 +423,5 @@ void PowerProfilesControl::updatePowerProfileHolds(QList<QVariantMap> holds)
 
     Q_EMIT profileHoldsChanged(m_profileHolds);
 }
+
+#include "moc_powerprofilescontrol.cpp"
