@@ -75,11 +75,27 @@ const QDBusArgument &operator>>(const QDBusArgument &argument, LogindInhibition
     return argument;
 }
 
+QDBusArgument &operator<<(QDBusArgument &argument, const PolicyAgentInhibition &inhibition)
+{
+    argument.beginStructure();
+    argument << inhibition.what << inhibition.who << inhibition.why << inhibition.mode << inhibition.flags;
+    argument.endStructure();
+    return argument;
+}
+
+const QDBusArgument &operator>>(const QDBusArgument &argument, PolicyAgentInhibition &inhibition)
+{
+    argument.beginStructure();
+    argument >> inhibition.what >> inhibition.who >> inhibition.why >> inhibition.mode >> inhibition.flags;
+    argument.endStructure();
+    return argument;
+}
+
 Q_DECLARE_METATYPE(NamedDBusObjectPath)
 Q_DECLARE_METATYPE(LogindInhibition)
 Q_DECLARE_METATYPE(QList<LogindInhibition>)
-Q_DECLARE_METATYPE(InhibitionInfo)
-Q_DECLARE_METATYPE(QList<InhibitionInfo>)
+Q_DECLARE_METATYPE(PolicyAgentInhibition)
+Q_DECLARE_METATYPE(QList<PolicyAgentInhibition>)
 
 namespace PowerDevil
 {
@@ -112,7 +128,6 @@ PolicyAgent *PolicyAgent::instance()
 
 PolicyAgent::PolicyAgent(QObject *parent)
     : QObject(parent)
-    , m_screenLockerWatcher(new QDBusServiceWatcher(this))
     , m_sdAvailable(false)
     , m_systemdInhibitFd(-1)
     , m_ckAvailable(false)
@@ -133,8 +148,11 @@ PolicyAgent::~PolicyAgent()
 
 void PolicyAgent::init(GlobalSettings *globalSettings)
 {
-    qDBusRegisterMetaType<InhibitionInfo>();
-    qDBusRegisterMetaType<QList<InhibitionInfo>>();
+    qDBusRegisterMetaType<PolicyAgentInhibition>();
+    qDBusRegisterMetaType<QList<PolicyAgentInhibition>>();
+
+    // DEPRECATED: for ListInhibitions, InhibitionsChanged
+    qDBusRegisterMetaType<QList<QStringList>>();
 
     // Watch over the systemd service
     m_sdWatcher.data()->setConnection(QDBusConnection::systemBus());
@@ -167,36 +185,28 @@ void PolicyAgent::init(GlobalSettings *globalSettings)
     connect(m_busWatcher.data(), &QDBusServiceWatcher::serviceUnregistered, this, &PolicyAgent::onServiceUnregistered);
 
     // Setup the screen locker watcher and check whether the screen is currently locked
-    connect(m_screenLockerWatcher, &QDBusServiceWatcher::serviceOwnerChanged, this, &PolicyAgent::onScreenLockerOwnerChanged);
-    m_screenLockerWatcher->setWatchMode(QDBusServiceWatcher::WatchForOwnerChange);
-    m_screenLockerWatcher->addWatchedService(SCREEN_LOCKER_SERVICE_NAME);
+    m_screenLockerInterface =
+        new OrgFreedesktopScreenSaverInterface(SCREEN_LOCKER_SERVICE_NAME, QStringLiteral("/ScreenSaver"), QDBusConnection::sessionBus(), this);
+    connect(m_screenLockerInterface, &OrgFreedesktopScreenSaverInterface::ActiveChanged, this, &PolicyAgent::onScreenLockerActiveChanged);
 
-    // async variant of QDBusConnectionInterface::serviceOwner ...
-    auto msg = QDBusMessage::createMethodCall(QStringLiteral("org.freedesktop.DBus"),
-                                              QStringLiteral("/"),
-                                              QStringLiteral("org.freedesktop.DBus"),
-                                              QStringLiteral("GetNameOwner"));
-    msg.setArguments({SCREEN_LOCKER_SERVICE_NAME});
-
-    auto *watcher = new QDBusPendingCallWatcher(QDBusConnection::sessionBus().asyncCall(msg), this);
-    QObject::connect(watcher, &QDBusPendingCallWatcher::finished, this, [this](QDBusPendingCallWatcher *watcher) {
-        QDBusPendingReply<QString> reply = *watcher;
-        if (!reply.isError()) {
-            onScreenLockerOwnerChanged(SCREEN_LOCKER_SERVICE_NAME, {}, reply.value());
-        }
-        watcher->deleteLater();
+    // Emit PropertiesChanged D-Bus signal for interface properties, because QDBus doesn't do that
+    connect(this, &PolicyAgent::dbusPropertiesChanged, this, [this](const QStringList &props) {
+        QDBusMessage signal =
+            QDBusMessage::createSignal("/org/kde/Solid/PowerManagement/PolicyAgent"_L1, "org.freedesktop.DBus.Properties"_L1, "PropertiesChanged"_L1);
+        signal << u"org.kde.Solid.PowerManagement.PolicyAgent"_s << QVariantMap() << props;
+        QDBusConnection::sessionBus().send(signal);
     });
 
     m_config = globalSettings;
-    // get comma separated list of colon separated app - reason inhibition pairs
-    // that have been configured to be blocked
+    // get comma separated list of colon separated "app:reason" inhibition pairs
+    // that have been configured to be suppressed
     QStringList blockedInhibitions = m_config->blockedInhibitions();
     std::ranges::transform(std::as_const(blockedInhibitions),
                            std::inserter(m_configuredToBlockInhibitions, m_configuredToBlockInhibitions.begin()),
                            [](const QString &inhibition) {
-                               const QString app = inhibition.section(QLatin1Char(':'), 0, 0);
-                               const QString reason = inhibition.section(QLatin1Char(':'), 1);
-                               return qMakePair(app, reason);
+                               QString who = inhibition.section(QLatin1Char(':'), 0, 0);
+                               QString why = inhibition.section(QLatin1Char(':'), 1);
+                               return qMakePair(std::move(who), std::move(why));
                            });
 }
 
@@ -291,7 +301,7 @@ void PolicyAgent::onSessionHandlerRegistered(const QString &serviceName)
 
         // block logind from handling time-based inhibitions
         setupSystemdInhibition();
-        // and then track logind's ihibitions, too
+        // and then track logind's inhibitions, too
         QDBusConnection::systemBus().connect(SYSTEMD_LOGIN1_SERVICE,
                                              SYSTEMD_LOGIN1_PATH,
                                              u"org.freedesktop.DBus.Properties"_s,
@@ -412,6 +422,33 @@ void PolicyAgent::onActiveSessionChanged(const QString &activeSession)
     }
 }
 
+PolicyAgent::RequiredPolicies PolicyAgent::policiesInLogindWhat(const QString &what) const
+{
+    const auto types = what.split(QLatin1Char(':'));
+
+    RequiredPolicies policies{};
+
+    if (types.contains("sleep"_L1)) {
+        policies |= InterruptSession;
+    }
+    if (types.contains("idle"_L1)) {
+        policies |= ChangeScreenSettings;
+    }
+    return policies;
+}
+
+QString PolicyAgent::logindWhatForPolicies(PolicyAgent::RequiredPolicies policies) const
+{
+    QStringList what;
+    if (policies & InterruptSession) {
+        what.append(u"sleep"_s);
+    }
+    if (policies & ChangeScreenSettings) {
+        what.append(u"idle"_s);
+    }
+    return what.join(":"_L1);
+}
+
 void PolicyAgent::checkLogindInhibitions()
 {
     qCDebug(POWERDEVIL) << "Checking logind inhibitions";
@@ -440,17 +477,7 @@ void PolicyAgent::checkLogindInhibitions()
                 continue;
             }
 
-            const auto types = activeInhibition.what.split(QLatin1Char(':'));
-
-            RequiredPolicies policies{};
-
-            if (types.contains(QLatin1String("sleep"))) {
-                policies |= InterruptSession;
-            }
-            if (types.contains(QLatin1String("idle"))) {
-                policies |= ChangeScreenSettings;
-            }
-
+            RequiredPolicies policies = policiesInLogindWhat(activeInhibition.what);
             if (!policies) {
                 continue;
             }
@@ -563,32 +590,6 @@ PolicyAgent::RequiredPolicies PolicyAgent::requirePolicyCheck(PolicyAgent::Requi
     return retpolicies;
 }
 
-void PolicyAgent::onScreenLockerOwnerChanged(const QString &serviceName, const QString &oldOwner, const QString &newOwner)
-{
-    Q_UNUSED(oldOwner);
-    if (serviceName != SCREEN_LOCKER_SERVICE_NAME) {
-        return;
-    }
-
-    delete m_screenLockerInterface;
-    m_screenLockerInterface = nullptr;
-    m_screenLockerActive = false;
-
-    if (!newOwner.isEmpty()) {
-        m_screenLockerInterface = new OrgFreedesktopScreenSaverInterface(newOwner, QStringLiteral("/ScreenSaver"), QDBusConnection::sessionBus(), this);
-        connect(m_screenLockerInterface, &OrgFreedesktopScreenSaverInterface::ActiveChanged, this, &PolicyAgent::onScreenLockerActiveChanged);
-
-        auto *activeReplyWatcher = new QDBusPendingCallWatcher(m_screenLockerInterface->GetActive());
-        connect(activeReplyWatcher, &QDBusPendingCallWatcher::finished, this, [this](QDBusPendingCallWatcher *watcher) {
-            QDBusPendingReply<bool> reply = *watcher;
-            if (!reply.isError()) {
-                onScreenLockerActiveChanged(reply.value());
-            }
-            watcher->deleteLater();
-        });
-    }
-}
-
 void PolicyAgent::onScreenLockerActiveChanged(bool active)
 {
     const auto oldPolicies = unavailablePolicies();
@@ -608,7 +609,11 @@ void PolicyAgent::onScreenLockerActiveChanged(bool active)
 
 uint PolicyAgent::addInhibitionWithExplicitDBusService(uint types, const QString &appName, const QString &reason, const QString &service)
 {
-    InhibitionInfo info = qMakePair(appName, reason);
+    const InhibitionInternals inhibition{
+        .policies = static_cast<PolicyAgent::RequiredPolicies>(types),
+        .who = appName,
+        .why = reason,
+    };
 
     ++m_lastCookie;
 
@@ -619,103 +624,72 @@ uint PolicyAgent::addInhibitionWithExplicitDBusService(uint types, const QString
         m_busWatcher.data()->addWatchedService(service);
     }
 
-    m_pendingInhibitions.append(cookie);
+    m_pendingInhibitions.insert(cookie);
 
     qCDebug(POWERDEVIL) << "Scheduling inhibition from" << service << appName << "with cookie" << cookie << "and reason" << reason;
 
     // wait 5s before actually enforcing the inhibition
     // there might be short interruptions (such as receiving a message) where an app might automatically
     // post an inhibition but we don't want the system to constantly wakeup because of this
-    QTimer::singleShot(5000, this, [this, cookie, service, reason, appName, types] {
-        qCDebug(POWERDEVIL) << "Enforcing inhibition from" << service << appName << "with cookie" << cookie << "and reason" << reason;
+    QTimer::singleShot(5000, this, [this, cookie, service, inhibition] {
+        qCDebug(POWERDEVIL) << "Enforcing inhibition from" << service << inhibition.who << "with cookie" << cookie << "and reason" << inhibition.why;
 
         if (!m_pendingInhibitions.contains(cookie)) {
             qCDebug(POWERDEVIL) << "By the time we wanted to enforce the inhibition it was already gone; discarding it";
             return;
         }
 
-        InhibitionInfo info = qMakePair(appName, reason);
-        m_cookieToAppName.insert(cookie, info);
+        m_cookieToInhibition.insert(cookie, inhibition);
 
-        // when inhibition gets blocked later, revoke it if it is still active
-        m_blockInhibitionConnections.insert(cookie,
-                                            connect(this,
-                                                    &PolicyAgent::blockInhibitionRequested,
-                                                    this,
-                                                    [this, cookie, service, appName, reason](const InhibitionInfo &blockedInfo, const bool permanently) {
-                                                        InhibitionInfo info = qMakePair(appName, reason);
-                                                        if (blockedInfo == info && m_activeInhibitions.contains(cookie)) {
-                                                            qCDebug(POWERDEVIL)
-                                                                << "Inhibition with cookie" << cookie << "from" << service << appName << "with reason" << reason
-                                                                << "was configured by the user to be blocked; releasing";
-
-                                                            ReleaseInhibition(cookie, true);
-
-                                                            if (m_blockedInhibitions.contains(cookie)) {
-                                                                // was already blocked but differently (temporarily vs permanently)
-                                                                m_blockedInhibitions.remove(cookie);
-                                                                if (m_configuredToBlockInhibitions.contains(info)) {
-                                                                    Q_EMIT PermanentlyBlockedInhibitionsChanged(QList<InhibitionInfo>(), {info});
-                                                                } else {
-                                                                    Q_EMIT TemporarilyBlockedInhibitionsChanged(QList<InhibitionInfo>(), {info});
-                                                                }
-                                                            }
-                                                            m_blockedInhibitions.insert(cookie);
-                                                            if (permanently) {
-                                                                if (!m_configuredToBlockInhibitions.contains(info)) {
-                                                                    // newly permaanently blocked, ie not reblocked after temporarily unblocked
-                                                                    Q_EMIT PermanentlyBlockedInhibitionsChanged({info}, QList<InhibitionInfo>());
-                                                                }
-                                                            } else {
-                                                                Q_EMIT TemporarilyBlockedInhibitionsChanged({info}, QList<InhibitionInfo>());
-                                                            }
-                                                        }
-                                                    }));
-
-        // when inhibition gets unblocked later, retroactively add it if it still exists
-        m_unblockInhibitionConnections.insert(
+        m_inhibitionAllowedChangedConnections.insert(
             cookie,
             connect(this,
-                    &PolicyAgent::unblockInhibitionRequested,
+                    &PolicyAgent::inhibitionAllowedChanged,
                     this,
-                    [this, cookie, service, appName, reason, types](const InhibitionInfo &unblockedInfo, const bool permanently) {
-                        InhibitionInfo info = qMakePair(appName, reason);
-                        if (unblockedInfo == info && m_blockedInhibitions.contains(cookie) && m_cookieToAppName.contains(cookie)) {
-                            qCDebug(POWERDEVIL) << "Inhibition with cookie" << cookie << "from" << service << appName << "with reason" << reason
-                                                << "was configured by the user to be unblocked; adding retroactively";
+                    [this, cookie, service, inhibition](const RequestedInhibitionId &requested, bool allowed) {
+                        const RequestedInhibitionId thisId = qMakePair(inhibition.who, inhibition.why);
+                        if (thisId != requested) {
+                            return;
+                        }
 
-                            addInhibitionTypeHelper(cookie, static_cast<PolicyAgent::RequiredPolicies>(types));
+                        // when inhibition gets blocked later, revoke it if it is still active
+                        if (!allowed && m_activeInhibitions.contains(cookie)) {
+                            qCDebug(POWERDEVIL) << "Inhibition with cookie" << cookie << "from" << service << inhibition.who << "with reason" << inhibition.why
+                                                << "was configured by the user to be blocked; releasing";
+
+                            // remove m_activeInhibitions[cookie], notify D-Bus properties, retain cookie
+                            ReleaseInhibition(cookie, true);
+                        }
+
+                        // when inhibition is allowed again later, retroactively add it if it still exists
+                        if (allowed && !m_activeInhibitions.contains(cookie) && m_cookieToInhibition.contains(cookie)) {
+                            qCDebug(POWERDEVIL) << "Inhibition with cookie" << cookie << "from" << service << inhibition.who << "with reason" << inhibition.why
+                                                << "was configured by the user to be allowed again; adding retroactively";
+
+                            addInhibitionTypeHelper(cookie, inhibition.policies);
                             m_activeInhibitions.insert(cookie);
-                            Q_EMIT InhibitionsChanged({{info}}, {});
-                            m_pendingInhibitions.removeOne(cookie);
-
-                            m_blockedInhibitions.remove(cookie);
-                            if (permanently) {
-                                Q_EMIT PermanentlyBlockedInhibitionsChanged(QList<InhibitionInfo>(), {info});
-                            } else {
-                                Q_EMIT TemporarilyBlockedInhibitionsChanged(QList<InhibitionInfo>(), {info});
-                            }
+                            Q_EMIT dbusPropertiesChanged({u"RequestedInhibitions"_s, u"ActiveInhibitions"_s});
+                            Q_EMIT InhibitionsChanged({{QStringList{inhibition.who, inhibition.why}}}, {});
                         }
                     }));
 
         // reject inhibitions configured as blocked
-        if (m_configuredToBlockInhibitions.contains(info)) {
-            qCDebug(POWERDEVIL) << "Inhibition with cookie" << cookie << "from" << service << appName << "with reason" << reason
+        const RequestedInhibitionId id = qMakePair(inhibition.who, inhibition.why);
+        m_pendingInhibitions.remove(cookie);
+        m_requestedInhibitions.append(cookie);
+
+        if (m_configuredToBlockInhibitions.contains(id)) {
+            qCDebug(POWERDEVIL) << "Inhibition with cookie" << cookie << "from" << service << inhibition.who << "with reason" << inhibition.why
                                 << "has been configured by the user to be blocked; rejecting";
 
-            m_pendingInhibitions.removeOne(cookie);
-            m_blockedInhibitions.insert(cookie);
-            Q_EMIT PermanentlyBlockedInhibitionsChanged({info}, QList<InhibitionInfo>());
+            Q_EMIT dbusPropertiesChanged({u"RequestedInhibitions"_s});
             return;
         }
 
-        addInhibitionTypeHelper(cookie, static_cast<PolicyAgent::RequiredPolicies>(types));
-
+        addInhibitionTypeHelper(cookie, inhibition.policies);
         m_activeInhibitions.insert(cookie);
-
-        Q_EMIT InhibitionsChanged({{info}}, {});
-
-        m_pendingInhibitions.removeOne(cookie);
+        Q_EMIT dbusPropertiesChanged({u"RequestedInhibitions"_s, u"ActiveInhibitions"_s});
+        Q_EMIT InhibitionsChanged({{QStringList{inhibition.who, inhibition.why}}}, {});
     });
 
     return cookie;
@@ -760,7 +734,7 @@ void PolicyAgent::addInhibitionTypeHelper(uint cookie, PolicyAgent::RequiredPoli
 
 void PolicyAgent::ReleaseInhibition(uint cookie, bool retainCookie)
 {
-    qCDebug(POWERDEVIL) << "Releasing inhibition with cookie " << cookie;
+    qCDebug(POWERDEVIL) << "Releasing inhibition with cookie" << cookie;
 
     QString service = m_cookieToBusService.take(cookie);
     if (!m_busWatcher.isNull() && !service.isEmpty() && !m_cookieToBusService.key(service)) {
@@ -768,40 +742,35 @@ void PolicyAgent::ReleaseInhibition(uint cookie, bool retainCookie)
         m_busWatcher.data()->removeWatchedService(service);
     }
 
-    if (m_pendingInhibitions.removeOne(cookie)) {
-        qCDebug(POWERDEVIL) << "It was only scheduled for inhibition but not enforced yet, just discarding it";
-        return;
-    }
-
-    m_activeInhibitions.remove(cookie);
-    Q_EMIT InhibitionsChanged(QList<InhibitionInfo>(), {{m_cookieToAppName.value(cookie).first}});
-
-    if (m_blockedInhibitions.remove(cookie)) {
-        qCDebug(POWERDEVIL) << "It was blocked, just discarding it";
-        InhibitionInfo info = m_cookieToAppName.take(cookie);
-        m_cookieToAppName.remove(cookie);
-        if (m_configuredToBlockInhibitions.contains(info)) {
-            Q_EMIT PermanentlyBlockedInhibitionsChanged(QList<InhibitionInfo>(), {info});
-        } else {
-            Q_EMIT TemporarilyBlockedInhibitionsChanged(QList<InhibitionInfo>(), {info});
-        }
+    if (m_pendingInhibitions.remove(cookie)) {
+        qCDebug(POWERDEVIL) << "Inhibition was only scheduled but not enforced yet, just discarding it";
         return;
     }
+    bool wasActive = m_activeInhibitions.remove(cookie);
 
     // Delete information about the inhibition,
-    // but not when it is being released because it has been blocked,
-    // we may need to restore it later
+    // except when it is just being suppressed, we may need to restore it later
+    InhibitionInternals inhibition = m_cookieToInhibition.value(cookie);
     if (!retainCookie) {
-        m_cookieToAppName.remove(cookie);
+        m_requestedInhibitions.removeOne(cookie);
+        m_cookieToInhibition.remove(cookie);
 
-        if (m_blockInhibitionConnections.contains(cookie)) {
-            disconnect(m_blockInhibitionConnections.take(cookie));
-        }
-        if (m_unblockInhibitionConnections.contains(cookie)) {
-            disconnect(m_unblockInhibitionConnections.take(cookie));
+        if (m_inhibitionAllowedChangedConnections.contains(cookie)) {
+            disconnect(m_inhibitionAllowedChangedConnections.take(cookie));
         }
     }
 
+    if (!wasActive && retainCookie) {
+        qCDebug(POWERDEVIL) << "Inhibition was already suppressed, but retaining cookie";
+        return;
+    } else if (!wasActive && !retainCookie) {
+        qCDebug(POWERDEVIL) << "Inhibition was already suppressed, just discarding it";
+        Q_EMIT dbusPropertiesChanged({u"RequestedInhibitions"_s});
+        return;
+    }
+    Q_EMIT dbusPropertiesChanged({u"RequestedInhibitions"_s, u"ActiveInhibitions"_s});
+    Q_EMIT InhibitionsChanged(QList<QStringList>(), {{inhibition.who}});
+
     // Look through all of the inhibition types
     bool notify = false;
     if (m_typesToCookie[ChangeScreenSettings].contains(cookie)) {
@@ -825,11 +794,45 @@ void PolicyAgent::ReleaseInhibition(uint cookie, bool retainCookie)
     }
 }
 
-QList<InhibitionInfo> PolicyAgent::ListInhibitions() const
+QList<QStringList> PolicyAgent::ListInhibitions() const
 {
-    return std::transform_reduce(m_activeInhibitions.constBegin(), m_activeInhibitions.constEnd(), QList<InhibitionInfo>(), std::plus<>(), [this](uint cookie) {
-        return QList<InhibitionInfo>{m_cookieToAppName.value(cookie)};
+    QList<QStringList> result;
+    std::ranges::transform(listActiveInhibitions(), std::back_inserter(result), [this](const PolicyAgentInhibition &inh) {
+        return QStringList{inh.who, inh.why};
     });
+    return result;
+}
+
+QList<PolicyAgentInhibition> PolicyAgent::listActiveInhibitions() const
+{
+    QList<PolicyAgentInhibition> result = listRequestedInhibitions();
+    erase_if(result, [this](const PolicyAgentInhibition &inh) {
+        return !(inh.flags & PolicyAgentInhibition::Active);
+    });
+    return result;
+}
+
+QList<PolicyAgentInhibition> PolicyAgent::listRequestedInhibitions() const
+{
+    QList<PolicyAgentInhibition> result;
+    std::ranges::transform(m_requestedInhibitions, std::back_inserter(result), [this](uint cookie) {
+        InhibitionInternals inhibition = m_cookieToInhibition.value(cookie);
+        PolicyAgentInhibition::Flags flags{};
+        if (!m_configuredToBlockInhibitions.contains(qMakePair(inhibition.who, inhibition.why))) {
+            // in the future, an inhibition may be tagged inactive despite being allowed (for other reasons)
+            flags |= PolicyAgentInhibition::Allowed;
+            flags |= PolicyAgentInhibition::Active;
+        }
+        return PolicyAgentInhibition{
+            .what = logindWhatForPolicies(inhibition.policies),
+            .who = inhibition.who,
+            .why = inhibition.why,
+            .mode = u"block"_s, // a "block"-type inhibition, not an inhibition that was suppressed
+                                // (this is why we avoid using "block" in the API in favor of "allow")
+            .flags = flags,
+        };
+    });
+    return result;
 }
 
 bool PolicyAgent::HasInhibition(/*PolicyAgent::RequiredPolicies*/ uint types)
@@ -839,7 +842,7 @@ bool PolicyAgent::HasInhibition(/*PolicyAgent::RequiredPolicies*/ uint types)
 
 void PolicyAgent::releaseAllInhibitions()
 {
-    const QList<uint> allCookies = m_cookieToAppName.keys();
+    const QList<uint> allCookies = m_cookieToInhibition.keys();
     for (uint cookie : allCookies) {
         ReleaseInhibition(cookie);
     }
@@ -873,87 +876,39 @@ void PolicyAgent::setupSystemdInhibition()
         qCWarning(POWERDEVIL) << "failed to inhibit systemd powersave handling";
 }
 
-void PolicyAgent::BlockInhibition(const QString &appName, const QString &reason, bool permanently)
+void PolicyAgent::SetInhibitionAllowed(const QString &appName, const QString &reason, bool allowed)
 {
-    InhibitionInfo info = qMakePair(appName, reason);
-    qCDebug(POWERDEVIL) << "Blocking inhibitions from" << appName << "with reason" << reason;
-    Q_EMIT blockInhibitionRequested(info, permanently);
-    if (!permanently) {
-        return;
+    const RequestedInhibitionId id = qMakePair(appName, reason);
+
+    if (allowed) {
+        qCDebug(POWERDEVIL) << "Allowing (reactivating) inhibitions from" << appName << "with reason" << reason;
+
+        if (!m_configuredToBlockInhibitions.contains(id)) {
+            qCDebug(POWERDEVIL) << "Inhibitions are already active, ignoring call.";
+            return;
+        }
+        m_configuredToBlockInhibitions.remove(id);
+    } else {
+        qCDebug(POWERDEVIL) << "Blocking inhibitions from" << appName << "with reason" << reason;
+
+        if (m_configuredToBlockInhibitions.contains(id)) {
+            qCDebug(POWERDEVIL) << "Inhibitions are already blocked, ignoring call.";
+            return;
+        }
+        m_configuredToBlockInhibitions.insert(id);
     }
 
-    if (m_configuredToBlockInhibitions.contains(qMakePair(appName, reason))) {
-        qCDebug(POWERDEVIL) << "Inhibitions from" << appName << "with reason" << reason << "are already blocked";
-        return;
-    }
-    qCDebug(POWERDEVIL) << "Adding inhibitions from" << appName << "with reason" << reason << "to list of permanently blocked inhibitions";
+    Q_EMIT inhibitionAllowedChanged(id, allowed);
 
-    m_configuredToBlockInhibitions.insert(qMakePair(appName, reason));
     QStringList configuredBlockedInhibitions;
     std::ranges::transform(std::as_const(m_configuredToBlockInhibitions),
                            std::back_inserter(configuredBlockedInhibitions),
-                           [](const QPair<QString, QString> &inhibition) {
-                               return inhibition.first + QLatin1Char(':') + inhibition.second;
+                           [](const RequestedInhibitionId &id) {
+                               return id.first + QLatin1Char(':') + id.second;
                            });
     m_config->setBlockedInhibitions(configuredBlockedInhibitions);
     m_config->save();
 }
-
-void PolicyAgent::UnblockInhibition(const QString &appName, const QString &reason, bool permanently)
-{
-    InhibitionInfo info = qMakePair(appName, reason);
-    qCDebug(POWERDEVIL) << "Unblocking inhibitions from" << appName << "with reason" << reason;
-    Q_EMIT unblockInhibitionRequested(info, permanently);
-    if (!permanently) {
-        return;
-    }
-
-    if (!m_configuredToBlockInhibitions.contains(qMakePair(appName, reason))) {
-        qCDebug(POWERDEVIL) << "Inhibitions from" << appName << "with reason" << reason << "are not blocked";
-        return;
-    }
-    qCDebug(POWERDEVIL) << "Removing inhibitions from" << appName << "with reason" << reason << "from list of permanently blocked inhibitions";
-
-    m_configuredToBlockInhibitions.remove(qMakePair(appName, reason));
-    QStringList configuredBlockedInhibitions;
-    std::ranges::transform(std::as_const(m_configuredToBlockInhibitions),
-                           std::back_inserter(configuredBlockedInhibitions),
-                           [](const QPair<QString, QString> &inhibition) {
-                               return inhibition.first + QLatin1Char(':') + inhibition.second;
-                           });
-    m_config->setBlockedInhibitions(configuredBlockedInhibitions);
-    m_config->save();
-}
-
-QList<InhibitionInfo> PolicyAgent::ListPermanentlyBlockedInhibitions() const
-{
-    QList<uint> permanentlyBlockedInhibitions;
-    std::copy_if(m_blockedInhibitions.begin(), m_blockedInhibitions.end(), std::back_inserter(permanentlyBlockedInhibitions), [this](uint cookie) {
-        return m_configuredToBlockInhibitions.contains(m_cookieToAppName.value(cookie));
-    });
-    return std::transform_reduce(permanentlyBlockedInhibitions.constBegin(),
-                                 permanentlyBlockedInhibitions.constEnd(),
-                                 QList<InhibitionInfo>(),
-                                 std::plus<>(),
-                                 [this](uint cookie) {
-                                     return QList<InhibitionInfo>{m_cookieToAppName.value(cookie)};
-                                 });
-}
-
-QList<InhibitionInfo> PolicyAgent::ListTemporarilyBlockedInhibitions() const
-{
-    QList<uint> temporarilyBlockedInhibitions;
-    std::copy_if(m_blockedInhibitions.begin(), m_blockedInhibitions.end(), std::back_inserter(temporarilyBlockedInhibitions), [this](uint cookie) {
-        return !m_configuredToBlockInhibitions.contains(m_cookieToAppName.value(cookie));
-    });
-    return std::transform_reduce(temporarilyBlockedInhibitions.constBegin(),
-                                 temporarilyBlockedInhibitions.constEnd(),
-                                 QList<InhibitionInfo>(),
-                                 std::plus<>(),
-                                 [this](uint cookie) {
-                                     return QList<InhibitionInfo>{m_cookieToAppName.value(cookie)};
-                                 });
-}
 } // namespace PowerDevil
 
 #include "moc_powerdevilpolicyagent.cpp"
