@@ -38,7 +38,19 @@ inline constexpr QLatin1StringView CONSOLEKIT_SERVICE("org.freedesktop.ConsoleKi
 inline constexpr QLatin1StringView CONSOLEKIT_MANAGER_PATH("/org/freedesktop/ConsoleKit/Manager");
 inline constexpr QLatin1StringView CONSOLEKIT_MANAGER_IFACE("org.freedesktop.ConsoleKit.Manager");
 
-using InhibitionInfo = QPair<QString, QString>;
+struct PolicyAgentInhibition {
+    enum Flag {
+        Active = 0x1,
+        Allowed = 0x2,
+    };
+    Q_DECLARE_FLAGS(Flags, Flag)
+
+    QString what;
+    QString who;
+    QString why;
+    QString mode;
+    uint flags;
+};
 
 struct LogindInhibition {
     QString what;
@@ -65,6 +77,9 @@ class POWERDEVILCORE_EXPORT PolicyAgent : public QObject, protected QDBusContext
     Q_CLASSINFO("D-Bus Interface", "org.kde.Solid.PowerManagement.PolicyAgent")
 
 public:
+    Q_PROPERTY(QList<PolicyAgentInhibition> RequestedInhibitions READ listRequestedInhibitions)
+    Q_PROPERTY(QList<PolicyAgentInhibition> ActiveInhibitions READ listActiveInhibitions)
+
     enum RequiredPolicy {
         None = 0,
         InterruptSession = 1,
@@ -88,25 +103,23 @@ public:
 
     void setupSystemdInhibition();
 
+    // DEPRECATED: D-Bus getter method, returns array of {who, why}. Use properties instead
+    QList<QStringList> ListInhibitions() const;
+
 public Q_SLOTS:
     // Exported slots
-    uint AddInhibition(uint types, const QString &appName, const QString &reason);
+    uint AddInhibition(uint types, const QString &who, const QString &why);
     void ReleaseInhibition(uint cookie, bool retainCookie = false);
-    QList<InhibitionInfo> ListInhibitions() const;
     bool HasInhibition(uint types);
 
     void releaseAllInhibitions();
 
-    void BlockInhibition(const QString &appName, const QString &reason, bool permanently);
-    void UnblockInhibition(const QString &appName, const QString &reason, bool permanently);
-    QList<InhibitionInfo> ListPermanentlyBlockedInhibitions() const;
-    QList<InhibitionInfo> ListTemporarilyBlockedInhibitions() const;
+    void SetInhibitionAllowed(const QString &who, const QString &why, bool allowed);
 
 Q_SIGNALS:
     // Exported signals
-    void InhibitionsChanged(const QList<InhibitionInfo> &added, const QStringList &removed);
-    void PermanentlyBlockedInhibitionsChanged(const QList<InhibitionInfo> &added, const QList<InhibitionInfo> &removed);
-    void TemporarilyBlockedInhibitionsChanged(const QList<InhibitionInfo> &added, const QList<InhibitionInfo> &removed);
+    void dbusPropertiesChanged(const QStringList &props);
+    void InhibitionsChanged(const QList<QStringList> &added, const QStringList &removed); // DEPRECATED: array of {who, why} in `added`
 
     void unavailablePoliciesChanged(PowerDevil::PolicyAgent::RequiredPolicies newpolicies);
     void sessionActiveChanged(bool active);
@@ -122,24 +135,35 @@ private Q_SLOTS:
     void onManagerPropertyChanged(const QString &ifaceName, const QVariantMap &changedProps, const QStringList &invalidatedProps);
 
 private:
+    using RequestedInhibitionId = QPair<QString, QString>; // {who, why}
+
+    struct InhibitionInternals {
+        RequiredPolicies policies;
+        QString who;
+        QString why;
+    };
+
     explicit PolicyAgent(QObject *parent = nullptr);
 
     void init(GlobalSettings *globalSettings);
 
     void addInhibitionTypeHelper(uint cookie, RequiredPolicies types);
 
+    RequiredPolicies policiesInLogindWhat(const QString &what) const;
+    QString logindWhatForPolicies(RequiredPolicies policies) const;
     void checkLogindInhibitions();
 
     // Screen locker integration
-    void onScreenLockerOwnerChanged(const QString &serviceName, const QString &oldOwner, const QString &newOwner);
-    QDBusServiceWatcher *m_screenLockerWatcher;
-
     void onScreenLockerActiveChanged(bool active);
     OrgFreedesktopScreenSaverInterface *m_screenLockerInterface = nullptr;
     bool m_screenLockerActive = false;
 
     // This function serves solely for fd.o connector
-    uint addInhibitionWithExplicitDBusService(uint types, const QString &appName, const QString &reason, const QString &service);
+    uint addInhibitionWithExplicitDBusService(uint types, const QString &who, const QString &why, const QString &service);
+
+    // D-Bus property getters
+    QList<PolicyAgentInhibition> listActiveInhibitions() const;
+    QList<PolicyAgentInhibition> listRequestedInhibitions() const;
 
     // used by systemd and ConsoleKit
     QScopedPointer<QDBusInterface> m_managerIface;
@@ -158,16 +182,15 @@ private:
     QPointer<QDBusInterface> m_ckSeatInterface;
     bool m_sessionIsBeingInterrupted;
 
-    QHash<uint, QPair<QString, QString>> m_cookieToAppName;
+    QHash<uint, InhibitionInternals> m_cookieToInhibition;
     QHash<uint, QString> m_cookieToBusService;
     QHash<RequiredPolicy, QList<uint>> m_typesToCookie;
 
     QHash<uint, LogindInhibition> m_logindInhibitions;
 
-    QList<int> m_pendingInhibitions;
+    QSet<uint> m_pendingInhibitions;
     QSet<uint> m_activeInhibitions;
-    QSet<uint> m_blockedInhibitions;
-    QSet<uint> m_pendingBlockedInhibitions;
+    QList<uint> m_requestedInhibitions;
 
     uint m_lastCookie;
 
@@ -178,16 +201,14 @@ private:
     bool m_wasLastActiveSession;
 
     GlobalSettings *m_config = nullptr;
-    QSet<InhibitionInfo> m_configuredToBlockInhibitions;
+    QSet<RequestedInhibitionId> m_configuredToBlockInhibitions;
 
-    QHash<uint, QMetaObject::Connection> m_blockInhibitionConnections;
-    QHash<uint, QMetaObject::Connection> m_unblockInhibitionConnections;
+    QHash<uint, QMetaObject::Connection> m_inhibitionAllowedChangedConnections;
 
     friend class Core;
     friend class FdoConnector;
 
 Q_SIGNALS:
-    void blockInhibitionRequested(InhibitionInfo info, bool permanently);
-    void unblockInhibitionRequested(InhibitionInfo info, bool permanently);
+    void inhibitionAllowedChanged(const RequestedInhibitionId &requested, bool allowed);
 };
 }
